function results = spm_mongo_neurosynth(varargin)
%%%%%%%%%%%%
%Lookup voxel activations with the neurosynth.org database
%%%%%%%%%%%%
%%REQUIRED: SPM(only tested on 8) must be installed.
%You can use this script in two ways: 
%1) Pass a filename of an analyze-format statmap, or
%2) Estimate an SPM model and load it into the workspace with the "Results" button
%(in order to produce an xSPM structure)
%The script will then scrape neurosynth.org for information about your activations 
%%%%%%%%%%%%
%OPTIONS
%%%%%%%%%%%%
%%-If you pass the string 'savedata', a csv will be saved for each coordinate 
%found in the neurosynth database. 
%%-If you pass the string 'search', the program will search for activations
%in a "searchlight" centered on the given voxel. Beware of greatly diminishing returns,
%both in terms of relevant information and computing time, with higher
%radii (1-3 is probably best).
%
%example 1A:
%spm_neurosynth(xSPM); %loaded via the spm 8 results interface
%example 1B:
%spm_neurosynth('my_statmap.hdr'); %note that img/nii also work
%example 2:
%spm_neurosynth(xSPM,'savedata');
%example 3:
%spm_neurosynth(xSPM,'savedata','search',1);
%%%%%%%%%%%%
%By Drew Linsley (thanks to Jordan Theriault for suggestions + Sylvain Fiedler for the csv routine)
%Updated 7/10/13 -- new searchlight procedure; database is now located at
%mongolab.com
%%%%%%%%%%%%

if ischar(varargin{1}),
    input_volume = varargin{1};
    head = spm_vol(input_volume);
    vol = spm_read_vols(head);
    xSPM.DIM = head.dim';
    mat = head.mat;
    [nx,ny,nz] = ind2sub(head.dim,find(vol>0));
    xSPM.Z = vol(vol>0);
    xSPM.XYZmm = mat*[nx(:,1) ny(:,1) nz(:,1) ones(numel(nx(:,1)),1)]';
    xSPM.XYZ = [nx,ny,nz]';
elseif isstruct(varargin{1}),
    xSPM = varargin{1};
else
    error('Pass the name of an analyze-format statmap or an xSPM structure')
end
string_in = cellfun(@isstr,varargin);
string_in = varargin(string_in);
num_in = cellfun(@isnumeric,varargin);
num_in = varargin(num_in);
if numel(string_in)>0,
    save_trigger = sum(cell2mat(cellfun(@(x) regexp(x,'savedata'),string_in,'UniformOutput',false)));
    if ~isempty(num_in),
    light_radius = cellfun(@(x) isempty(regexp(x,'search')),string_in);
    light_radius = num_in{1};
    end
else
    save_trigger = 0;
    light_radius = 0;
end

if ~exist('light_radius','var'),
    light_radius = 0;
end

database_url = 'https://api.mongolab.com/api/1/databases/scenerater/collections/neurosynth';
api_key = '&apiKey=5001c9eae4b0e762af01b23a';

if light_radius > 0,
    fprintf('\r\rPositioning Searchlights...\r\r')
    search_vol = zeros(xSPM.DIM');
    [columnsInImage, rowsInImage, pagesInImage] = meshgrid(1:numel(search_vol(1,:,1)), 1:numel(search_vol(:,1,1)),1:numel(search_vol(1,1,:)));
    for idx = 1:numel(xSPM.XYZ(1,:)),
        for search_size = light_radius:-1:1,
            new_light = logical((rowsInImage - xSPM.XYZ(1,idx)).^2 + (columnsInImage - xSPM.XYZ(2,idx)).^2 + ...
                (pagesInImage - xSPM.XYZ(3,idx)).^2 <= search_size.^2);
            search_vol(new_light) = search_size;
        end
    end
    search_ind = find(search_vol>0);
    [nx,ny,nz] = ind2sub(xSPM.DIM',search_ind);
    vox_dists = search_vol(search_ind)-1;
    xSPM.XYZmm = mat*[nx(:,1) ny(:,1) nz(:,1) ones(numel(nx(:,1)),1)]';
    xSPM.XYZ = [nx,ny,nz]';
else
    vox_dists = zeros(numel(xSPM.XYZ(1,:)),1);
end
fprintf('\r\rAccessing neurosynth database (this may take a while)...\r\r')
results = cell(numel(xSPM.XYZmm(1,:)),6);
results(:,1) = arrayfun(@(x){x},xSPM.XYZmm(1,:));
results(:,2) = arrayfun(@(x){x},xSPM.XYZmm(2,:));
results(:,3) = arrayfun(@(x){x},xSPM.XYZmm(3,:));
results(:,6) = arrayfun(@(x){x},vox_dists);
status_backspace = arrayfun(@(x) numel(num2str(x)),1:numel(xSPM.XYZmm(1,:)));
status_backspace = [0 diff(status_backspace)*-1] + status_backspace;
status_backspace = arrayfun(@(x) repmat('\b',1,x),status_backspace,'UniformOutput', false);

good_activations = 0;
for idx = 1:numel(xSPM.XYZmm(1,:)),
    scrape_here = strcat(database_url,sprintf('?q={"x":%i,"y":%i,"z":%i}',xSPM.XYZmm(1,idx),xSPM.XYZmm(2,idx),xSPM.XYZmm(3,idx)),api_key);
        buffer = java.io.BufferedReader(java.io.InputStreamReader(openStream(java.net.URL(scrape_here))));
        buff_line = char(readLine(buffer));
    if strcmp(buff_line,'[  ]'),
        results{idx,4} = 'no data';
        results{idx,5} = 'no data';
    else
        region = regexp(buff_line,'"Region" : "','split');
        if numel(region) > 1,
            region = regexp(region{2},'" ,','split');
            results{idx,4} = region{1};
        else
            results{idx,4} = 'no anat. info';
        end
        statistic = regexp(buff_line,'"Statistic" : ','split');
        if numel(statistic) > 1,
            statistic = regexp(statistic{2},' ,','split');
            results{idx,5} = statistic{1};
        else
            results{idx,4} = 'no stat info';
        end
        good_activations = good_activations + 1;
    end
    fprintf(sprintf('%s%i',status_backspace{idx},idx))
end
header = cell(1,6);
header{1} = 'MNI X';
header{2} = 'MNI Y';
header{3} = 'MNI Z';
header{4} = 'Anatomical Label';
header{5} = 'Statistic';
header{6} = 'mm from activation?';
csv_activations = [header;results];

if save_trigger > 0,
    csv_name = sprintf('results_for_%i_activations.csv',good_activations);
    cell2csv(csv_name,csv_activations);
    fprintf('\rOutput written as %s to %s \r',csv_name,pwd)
end


function cell2csv(fileName, cellArray, separator, excelYear, decimal)
% Writes cell array content into a *.csv file.
%
% CELL2CSV(fileName, cellArray, separator, excelYear, decimal)
%
% fileName     = Name of the file to save. [ i.e. 'text.csv' ]
% cellArray    = Name of the Cell Array where the data is in
% separator    = sign separating the values (default = ';')
% excelYear    = depending on the Excel version, the cells are put into
%                quotes before they are written to the file. The separator
%                is set to semicolon (;)
% decimal      = defines the decimal separator (default = '.')
%
%         by Sylvain Fiedler, KA, 2004
% updated by Sylvain Fiedler, Metz, 06
% fixed the logical-bug, Kaiserslautern, 06/2008, S.Fiedler
% added the choice of decimal separator, 11/2010, S.Fiedler

%% Checking fr optional Variables
if ~exist('separator', 'var')
    separator = ',';
end

if ~exist('excelYear', 'var')
    excelYear = 1997;
end

if ~exist('decimal', 'var')
    decimal = '.';
end

%% Setting separator for newer excelYears
if excelYear > 2000
    separator = ';';
end

%% Write file
datei = fopen(fileName, 'w');

for z=1:size(cellArray, 1)
    for s=1:size(cellArray, 2)
        
        var = eval(['cellArray{z,s}']);
        % If zero, then empty cell
        if size(var, 1) == 0
            var = '';
        end
        % If numeric -> String
        if isnumeric(var)
            var = num2str(var);
            % Conversion of decimal separator (4 Europe & South America)
            % http://commons.wikimedia.org/wiki/File:DecimalSeparator.svg
            if decimal ~= '.'
                var = strrep(var, '.', decimal);
            end
        end
        % If logical -> 'true' or 'false'
        if islogical(var)
            if var == 1
                var = 'TRUE';
            else
                var = 'FALSE';
            end
        end
        % If newer version of Excel -> Quotes 4 Strings
        if excelYear > 2000
            var = ['"' var '"'];
        end
        
        % OUTPUT value
        fprintf(datei, '%s', var);
        
        % OUTPUT separator
        if s ~= size(cellArray, 2)
            fprintf(datei, separator);
        end
    end
    if z ~= size(cellArray, 1) % prevent a empty line at EOF
        % OUTPUT newline
        fprintf(datei, '\n');
    end
end
% Closing file
fclose(datei);
% END


